@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
- * Copyright 2002-2010 by Ping Cheng, Wacom. <pingc@wacom.com>
+ * Copyright 2002-2013 by Ping Cheng, Wacom. <pingc@wacom.com>
  *                                                                            
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -396,27 +396,24 @@ static void wcmLinkTouchAndPen(InputInfoPtr pInfo)
 	InputInfoPtr device = xf86FirstLocalDevice();
 	WacomCommonPtr tmpcommon = NULL;
 	WacomDevicePtr tmppriv = NULL;
-	Bool touch_device_assigned = FALSE;
 
-	/* Lookup to find the associated pen and touch */
+	/* Lookup to find the associated pen and touch with same product id */
 	for (; device != NULL; device = device->next)
 	{
 		if (!strcmp(device->drv->driverName, "wacom"))
 		{
 			tmppriv = (WacomDevicePtr) device->private;
 			tmpcommon = tmppriv->common;
-			touch_device_assigned = (common->wcmTouchDevice ||
-						tmpcommon->wcmTouchDevice);
 
 			/* skip the same tool or already linked devices */
-			if ((tmppriv == priv) || touch_device_assigned)
+			if ((tmppriv == priv) || tmpcommon->wcmTouchDevice)
 				continue;
 
 			if (tmpcommon->tablet_id == common->tablet_id)
 			{
-				if (IsTouch(tmppriv) && IsPen(priv))
+				if (IsTouch(tmppriv) && IsTablet(priv))
 					common->wcmTouchDevice = tmppriv;
-				else if (IsTouch(priv) && IsPen(tmppriv))
+				else if (IsTouch(priv) && IsTablet(tmppriv))
 					tmpcommon->wcmTouchDevice = priv;
 
 				if (common->wcmTouchDevice ||
@@ -426,6 +423,37 @@ static void wcmLinkTouchAndPen(InputInfoPtr pInfo)
 					TabletSetFeature(tmpcommon, WCM_PENTOUCH);
 				}
 			}
+
+			if (common->wcmTouchDevice)
+				return;
+		}
+	}
+
+	/* Lookup for pen and touch devices with different product ids */
+	for (; device != NULL; device = device->next)
+	{
+		if (!strcmp(device->drv->driverName, "wacom"))
+		{
+			tmppriv = (WacomDevicePtr) device->private;
+			tmpcommon = tmppriv->common;
+
+			/* skip the same tool or already linked devices */
+			if ((tmppriv == priv) || tmpcommon->wcmTouchDevice)
+				continue;
+
+			if (IsTouch(tmppriv) && IsTablet(priv))
+				common->wcmTouchDevice = tmppriv;
+			else if (IsTouch(priv) && IsTablet(tmppriv))
+				tmpcommon->wcmTouchDevice = priv;
+
+			if (common->wcmTouchDevice || tmpcommon->wcmTouchDevice)
+			{
+				TabletSetFeature(common, WCM_PENTOUCH);
+				TabletSetFeature(tmpcommon, WCM_PENTOUCH);
+			}
+
+			if (common->wcmTouchDevice)
+				return;
 		}
 	}
 }
@@ -441,8 +469,10 @@ static void wcmLinkTouchAndPen(InputInfoPtr pInfo)
  */
 static int wcmIsHotpluggedDevice(InputInfoPtr pInfo)
 {
-	char *source = xf86CheckStrOption(pInfo->options, "_source", NULL);
-	return (!source || !strcmp(source, "_driver/wacom"));
+	char *source = xf86CheckStrOption(pInfo->options, "_source", "");
+	int matches = (strcmp(source, "_driver/wacom") == 0);
+	free(source);
+	return matches;
 }
 
 /* wcmPreInit - called for each input devices with the driver set to
@@ -482,8 +512,8 @@ static int wcmPreInit(InputDriverPtr drv, InputInfoPtr pInfo, int flags)
 {
 	WacomDevicePtr priv = NULL;
 	WacomCommonPtr common = NULL;
-	const char*	type;
-	const char*	device, *oldname;
+	char		*type, *device;
+	char		*oldname = NULL;
 	int		need_hotplug = 0, is_dependent = 0;
 
 	gWacomModule.wcmDrv = drv;
@@ -533,7 +563,7 @@ static int wcmPreInit(InputDriverPtr drv, InputInfoPtr pInfo, int flags)
 
 	common->debugLevel = xf86SetIntOption(pInfo->options,
 					      "CommonDBG", common->debugLevel);
-	oldname = pInfo->name;
+	oldname = strdup(pInfo->name);
 
 	if (wcmIsHotpluggedDevice(pInfo))
 		is_dependent = 1;
@@ -544,6 +574,7 @@ static int wcmPreInit(InputDriverPtr drv, InputInfoPtr pInfo, int flags)
 		char *new_name;
 		if (asprintf(&new_name, "%s %s", pInfo->name, type) == -1)
 			new_name = strdup(pInfo->name);
+		free(pInfo->name);
 		pInfo->name = priv->name = new_name;
 	}
 
@@ -575,12 +606,15 @@ static int wcmPreInit(InputDriverPtr drv, InputInfoPtr pInfo, int flags)
 		pInfo->fd = -1;
 	}
 
-	/* only link them once per port. We need to try for both pen and touch
-	 * since we do not know which tool (touch or pen) will be added first.
+	/* only link them once per port. We need to try for both tablet tool
+	 * and touch since we do not know which tool will be added first.
 	 */
-	if (IsTouch(priv) || (IsPen(priv) && !common->wcmTouchDevice))
+	if (IsTouch(priv) || (IsTablet(priv) && !common->wcmTouchDevice))
 		wcmLinkTouchAndPen(pInfo);
 
+	free(type);
+	free(oldname);
+
 	return Success;
 
 SetupProc_fail:
@@ -594,6 +628,8 @@ SetupProc_fail:
 		pInfo->fd = -1;
 	}
 
+	free(type);
+	free(oldname);
 	return BadMatch;
 }
 
