@@ -1,5 +1,5 @@
 /*
- * Copyright 2009 - 2010 by Ping Cheng, Wacom. <pingc@wacom.com>
+ * Copyright 2009 - 2013 by Ping Cheng, Wacom. <pingc@wacom.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,12 +35,17 @@ static Bool wcmCheckSource(InputInfoPtr pInfo, dev_t min_maj)
 	int match = 0;
 	InputInfoPtr pDevices = xf86FirstLocalDevice();
 
-	for (; pDevices != NULL; pDevices = pDevices->next)
+	for (; !match && pDevices != NULL; pDevices = pDevices->next)
 	{
 		char* device = xf86CheckStrOption(pDevices->options, "Device", NULL);
 
 		/* device can be NULL on some distros */
-		if (!device || !strstr(pDevices->drv->driverName, "wacom"))
+		if (!device)
+			continue;
+
+		free(device);
+
+		if (!strstr(pDevices->drv->driverName, "wacom"))
 			continue;
 
 		if (pInfo != pDevices)
@@ -55,11 +60,10 @@ static Bool wcmCheckSource(InputInfoPtr pInfo, dev_t min_maj)
 				/* only add the new tool if the matching major/minor
 				* was from the same source */
 				if (strcmp(fsource, psource))
-				{
 					match = 1;
-					break;
-				}
 			}
+			free(fsource);
+			free(psource);
 		}
 	}
 	if (match)
@@ -112,6 +116,7 @@ int wcmIsDuplicate(const char* device, InputInfoPtr pInfo)
 		isInUse = 4;
 	}
 ret:
+	free(lsource);
 	return isInUse;
 }
 
@@ -134,7 +139,7 @@ Bool wcmIsAValidType(InputInfoPtr pInfo, const char* type)
 	int j, k, ret = FALSE;
 	WacomDevicePtr priv = (WacomDevicePtr)pInfo->private;
 	WacomCommonPtr common = priv->common;
-	char* dsource = xf86CheckStrOption(pInfo->options, "_source", NULL);
+	char* dsource;
 
 	if (!type)
 	{
@@ -142,6 +147,8 @@ Bool wcmIsAValidType(InputInfoPtr pInfo, const char* type)
 		return FALSE;
 	}
 
+	dsource = xf86CheckStrOption(pInfo->options, "_source", NULL);
+
 	/* walkthrough all types */
 	for (j = 0; j < ARRAY_SIZE(wcmType); j++)
 	{
@@ -175,6 +182,7 @@ Bool wcmIsAValidType(InputInfoPtr pInfo, const char* type)
 		xf86Msg(X_ERROR, "%s: Invalid type '%s' for this device.\n",
 			pInfo->name, type);
 
+	free(dsource);
 	return ret;
 }
 
@@ -255,6 +263,7 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 		case 0xC6:  /* CintiqV5 */
 		case 0xCC:  /* CinitqV5 */
 		case 0xFA:  /* Cintiq 22HD */
+		case 0x5B:  /* Cintiq 22HDT Pen */
 			TabletSetFeature(priv->common, WCM_LCD);
 			/* fall through */
 		case 0xB0:  /* I3 */
@@ -277,6 +286,7 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 		case 0xE6: /* TPC with 2FGT */
 		case 0x93: /* TPC with 1FGT */
 		case 0x9A: /* TPC with 1FGT */
+		case 0xEC: /* TPC with 1FGT */
 		case 0xED: /* TPC with 1FGT */
 		case 0x90: /* TPC */
 		case 0x97: /* TPC */
@@ -286,6 +296,11 @@ int wcmDeviceTypeKeys(InputInfoPtr pInfo)
 
 		case 0x9F:
 		case 0xF6: /* Cintiq 24HDT Touch */
+		case 0x57: /* DTK2241 */
+		case 0x59: /* DTH2242 Pen */
+		case 0x5D: /* DTH2242 Touch */
+		case 0x5E: /* Cintiq 22HDT Touch */
+		case 0x304:/* Cintiq 13HD */
 			TabletSetFeature(priv->common, WCM_LCD);
 			break;
 	}
@@ -367,7 +382,7 @@ static InputOption *wcmOptionDupConvert(InputInfoPtr pInfo, const char* basename
 	WacomToolPtr ser = common->serials;
 	InputOption *iopts = NULL;
 	char *name;
-	pointer options;
+	pointer options, o;
 	int rc;
 
 #if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 12
@@ -405,13 +420,15 @@ static InputOption *wcmOptionDupConvert(InputInfoPtr pInfo, const char* basename
 
 	free(name);
 
-	while(options)
+	o = options;
+	while(o)
 	{
 		iopts = input_option_new(iopts,
-					 xf86OptionName(options),
-					 xf86OptionValue(options));
-		options = xf86NextOption(options);
+					 xf86OptionName(o),
+					 xf86OptionValue(o));
+		o = xf86NextOption(o);
 	}
+	xf86OptionListFree(options);
 	return iopts;
 }
 
@@ -427,10 +444,12 @@ static InputAttributes* wcmDuplicateAttributes(InputInfoPtr pInfo,
 {
 	int rc;
 	InputAttributes *attr;
+	char *product;
+
 	attr = DuplicateInputAttributes(pInfo->attrs);
-	rc = asprintf(&attr->product, "%s %s", attr->product, type);
-	if (rc == -1)
-		attr->product = NULL;
+	rc = asprintf(&product, "%s %s", attr->product, type);
+	free(attr->product);
+	attr->product = (rc != -1) ? product : NULL;
 	return attr;
 }
 #endif
@@ -575,19 +594,20 @@ void wcmHotplugOthers(InputInfoPtr pInfo, const char *basename)
  * This changes the source to _driver/wacom, all auto-hotplugged devices
  * will have the same source.
  */
-int wcmNeedAutoHotplug(InputInfoPtr pInfo, const char **type)
+int wcmNeedAutoHotplug(InputInfoPtr pInfo, char **type)
 {
 	char *source = xf86CheckStrOption(pInfo->options, "_source", NULL);
 	int i;
+	int rc = 0;
 
 	if (*type) /* type specified, don't hotplug */
-		return 0;
+		goto out;
 
 	if (!source) /* xorg.conf device, don't auto-pick type */
-		return 0;
+		goto out;
 
 	if (source && strcmp(source, "server/hal") && strcmp(source, "server/udev"))
-		return 0;
+		goto out;
 
 	/* no type specified, so we need to pick the first one applicable
 	 * for our device */
@@ -595,13 +615,14 @@ int wcmNeedAutoHotplug(InputInfoPtr pInfo, const char **type)
 	{
 		if (wcmIsAValidType(pInfo, wcmType[i].type))
 		{
+			free(*type);
 			*type = strdup(wcmType[i].type);
 			break;
 		}
 	}
 
 	if (!*type)
-		return 0;
+		goto out;
 
 	xf86Msg(X_INFO, "%s: type not specified, assuming '%s'.\n", pInfo->name, *type);
 	xf86Msg(X_INFO, "%s: other types will be automatically added.\n", pInfo->name);
@@ -610,7 +631,11 @@ int wcmNeedAutoHotplug(InputInfoPtr pInfo, const char **type)
 	pInfo->options = xf86AddNewOption(pInfo->options, "Type", *type);
 	pInfo->options = xf86ReplaceStrOption(pInfo->options, "_source", "_driver/wacom");
 
-	return 1;
+	rc = 1;
+
+	free(source);
+out:
+	return rc;
 }
 
 int wcmParseSerials (InputInfoPtr pInfo)
@@ -737,6 +762,8 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 		 */
 	}
 
+	free(s);
+
 	/* Pad is always in absolute mode.
 	 * The pad also defaults to wheel scrolling, unlike the pens
 	 * (interesting effects happen on ArtPen and others with build-in
@@ -773,6 +800,7 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 					" device\n", pInfo->name);
 		else
 			wcmRotateTablet(pInfo, rotation);
+		free(s);
 	}
 
 	common->wcmRawSample = xf86SetIntOption(pInfo->options, "RawSample",
@@ -794,10 +822,6 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 			common->wcmSuppress = DEFAULT_SUPPRESS;
 	}
 
-	if (xf86SetBoolOption(pInfo->options, "Tilt",
-			(common->wcmFlags & TILT_REQUEST_FLAG)))
-		common->wcmFlags |= TILT_REQUEST_FLAG;
-
 	/* pressure curve takes control points x1,y1,x2,y2
 	 * values in range from 0..100.
 	 * Linear curve is 0,0,100,100
@@ -815,6 +839,7 @@ Bool wcmPreInitParseOptions(InputInfoPtr pInfo, Bool is_primary,
 		else
 			wcmSetPressureCurve(priv,a,b,c,d);
 	}
+	free(s);
 
 	/*Serials of tools we want hotpluged*/
 	if (wcmParseSerials (pInfo) != 0)
