@@ -570,6 +570,43 @@ static struct modifier specialkeys[] = {
 
 	{"PgUp", "Prior"}, {"PgDn", "Next"},
 
+	{"del", "Delete"}, {"home", "Home"},
+	{"end", "End"},
+
+	{"`", "quoteleft"},
+	{"-", "minus"},
+	{"=", "equal"},
+	{"[", "bracketleft"},
+	{"]", "bracketright"},
+	{"\\", "backslash"},
+	{";", "semicolon"},
+	{"'", "apostrophe"},
+	{",", "comma"},
+	{".", "period"},
+	{"/", "slash"},
+
+	{"~", "asciitilde"},
+	{"!", "exclam"},
+	{"@", "at"},
+	{"#", "numbersign"},
+	{"$", "dollar"},
+	{"%", "percent"},
+	{"^", "asciicircum"},
+	{"&", "ampersand"},
+	{"*", "asterisk"},
+	{"(", "parenleft"},
+	{")", "parenright"},
+	{"_", "underscore"},
+	{"+", "plus"},
+	{"{", "braceleft"},
+	{"}", "braceright"},
+	{"|", "bar"},
+	{":", "colon"},
+	{"\"", "quotedbl"},
+	{"<", "less"},
+	{">", "greater"},
+	{"?", "question"},
+
 	{ NULL, NULL }
 };
 
@@ -1012,18 +1049,17 @@ static int keysym_to_keycode(Display *dpy, KeySym sym)
 
 	for (kc = xkb->min_key_code; kc <= xkb->max_key_code; kc++)
 	{
-		KeySym* ks;
 		int i;
 
-		ks = XkbKeySymsPtr(xkb, kc);
 		for (i = 0; i < XkbKeyGroupWidth(xkb, kc, state.group); i++)
-			if (ks[i] == sym)
+			if (XKeycodeToKeysym(dpy, kc, i) == sym)
 				goto out;
 	}
 
 out:
 	return kc;
 }
+
 /*
    Map gibberish like "ctrl alt f2" into the matching AC_KEY values.
    Returns 1 on success or 0 otherwise.
@@ -1076,7 +1112,15 @@ static int special_map_keystrokes(Display *dpy, int argc, char **argv, unsigned
 		} else
 			need_press = need_release = 1;
 
+
 		ks = XStringToKeysym(key);
+		if (ks == 0) {
+			key = convert_specialkey(key);
+			ks = XStringToKeysym(key);
+			if (ks == 0)
+				fprintf(stderr, "Warning: unable to map '%s' to a keycode.\n", key);
+		}
+
 		kc = keysym_to_keycode(dpy, ks);
 
 		if (need_press)
@@ -1216,9 +1260,10 @@ static Bool parse_actions(Display *dpy, int argc, char **argv, unsigned long* da
  * the driver can understand.
  *
  * Once we have a list of actions, we can store it in the appropriate
- * child property. If none exists, we must first create one and update
- * the parent list. If we want no action to occur, we can delete the
- * child property and have the parent point to '0' instead.
+ * child property. Action atoms should be pre-created by the server,
+ * so it is an error if one does not exist. To reset the action to its
+ * default, we can have the parent point to '0' (which is a special
+ * signal to the Wacom driver).
  *
  * @param  dpy         X display we want to query
  * @param  dev         X device we want to modify
@@ -1259,16 +1304,20 @@ static void special_map_property(Display *dpy, XDevice *dev, Atom btnact_prop, i
 
 	/* set or unset the property */
 	prop = btnact_data[offset];
-	if (nitems > 0)
-	{ /* Setting a new or existing property */
-		if (!prop)
-		{
-			char buff[64];
-			sprintf(buff, "%s action %d", XGetAtomName(dpy, btnact_prop), (offset + 1));
-			prop = XInternAtom(dpy, buff, False);
-			btnact_data[offset] = prop;
-		}
+	if (!prop)
+	{
+		/* The subproperty at the given offset is set to 'None',
+		 * meaning the device does not support its meaning. E.g.
+		 * buttons 4-7 are 'None' since the device doesn't have
+		 * physical buttons relating to them.
+		 */
+		fprintf(stderr, "Unsupported offset into '%s' property.\n",
+			XGetAtomName(dpy, btnact_prop));
+		goto out;
+	}
 
+	if (nitems > 0)
+	{ /* Setting an existing property */
 		/* FIXME: the property containing the key sequence must be
 		 * set before updating the button action properties */
 		XChangeDeviceProperty(dpy, dev, prop, XA_INTEGER, 32,
@@ -1288,8 +1337,6 @@ static void special_map_property(Display *dpy, XDevice *dev, Atom btnact_prop, i
 					PropModeReplace,
 					(unsigned char*)btnact_data,
 					btnact_nitems);
-
-		XDeleteDeviceProperty(dpy, dev, prop);
 	}
 
 	XFlush(dpy);
@@ -2674,15 +2721,36 @@ static void test_convert_specialkey(void)
 	char buff[5];
 	struct modifier *m;
 
-	/* make sure at least the default keys (ascii 33 - 126) aren't
-	 * specialkeys */
-	for (i = '!'; i <= '~'; i++)
+	/* make sure a-zA-Z aren't specialkeys */
+	for (i = 'a'; i <= 'z'; i++)
 	{
 		sprintf(buff, "%c", i);
 		converted = convert_specialkey(buff);
 		assert(strcmp(converted, buff) == 0);
 	}
 
+	for (i = 'A'; i <= 'Z'; i++)
+	{
+		sprintf(buff, "%c", i);
+		converted = convert_specialkey(buff);
+		assert(strcmp(converted, buff) == 0);
+	}
+
+	/* punctuation are specialkeys */
+	for (i = '!'; i <= '/'; i++)
+	{
+		sprintf(buff, "%c", i);
+		converted = convert_specialkey(buff);
+		assert(strcmp(converted, buff) != 0);
+	}
+	for (i = ':'; i <= '?'; i++)
+	{
+		sprintf(buff, "%c", i);
+		converted = convert_specialkey(buff);
+		assert(strcmp(converted, buff) != 0);
+	}
+
+
 	for (m = specialkeys; m->name; m++)
 	{
 		converted = convert_specialkey(m->name);
